window.Gleam.initSearch([{"doc":"signal","title":"signal","content":"[![Package Version](https://img.shields.io/hexpm/v/signal)](https://hex.pm/packages/signal)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/signal/)\n\n&lt;h1 align=&quot;center&quot;&gt;Signal&lt;/h1&gt;\n\n&lt;div align=&quot;center&quot;&gt;\n    Makes event sourcing a piece of cake! 🍰\n&lt;/div&gt;\n\n---\n\nEvent sourcing is a software design pattern where the state of an application is determined by a sequence of events. It differs from traditional software modeling by storing and replaying events to derive the current state, rather than directly modifying the state through mutable operations.\n\nIn event sourcing, we process **commands**, which, informed by a **model**, output **events**. The events are then applied to the model to produce new state. The events are persisted instead of the model itself, allowing for an auditable, append only storage model that supports history, rollbacks, and generally avoids need for db migrations.\n\n&gt; **Command** -&gt; produces -&gt; **Events** -&gt; mutates -&gt; **Model**\n\nThe model is often referred to as an Aggregate, inspired by the Domain Driven Design approach.\n\nIt can make your applications very easy to reason about and extend.\n\n```sh\ngleam add signal\n```\n\n## Features\n\n-   A **declarative API** that does not intrude into your domain model\n-   **Effortless extensibility** allowing for custom projections, metrics and any other kind of compute to be triggered on event generation\n-   High degree of **scalability** harnessing the power of OTP\n\n## Example\n\n### Creating an aggregate\n\nAggregates are identified by a unique string id, and can then be retrieved using that id.\n\n```gleam\nuse cart &lt;- result.try(signal.create(signal, &quot;new_unique_cart_id&quot;))\n```\n\n### Processing a command\n\nThis will run your command on a given aggregate, which may produce an event resulting in a new state.\n\n```gleam\ncase signal.handle_command(cart, domain.CompletePurchase) {\n    Ok(new_state) -&gt; todo\n    Error(msg) -&gt; todo\n}\n```\n\n### Get aggregate state\n\nThis will get the state for a given aggregate. Signal manages a pool of in-memory aggregates to improve performance, if an aggregate is not in the pool, it will get events from the storage layer and derive the state.\n\n```gleam\nuse aggregate &lt;- result.try(signal.aggregate(signal, &quot;id_of_aggregate&quot;))\nlet state = signal.get_state(aggregate)\n```\n\n## Learning Signal\n\n[Cart example](https://github.com/dominikbb/signal/tree/master/examples/cart) creates a web app using Signal, Wisp and HTMX.\n\n## Road to v1\n\nThese are the features I see version 1 having:\n\n-   [x] Logging handler for customizing logging\n-   [x] Persistance layer test suite\n-   [x] Postgres persistance layer\n-   [ ] Implement a write ahead log for batching event storage\n-   [ ] Aggregate replay to a specific version\n-   [ ] Genesis command handler for customizing creation of a new aggregate\n-   [ ] Closure event handler, to make deleting and evicting aggregates more customizable\n-   [ ] Aggregate pool management based on activity instead of a queue\n-   [ ] Test and develop policy subscribers further\n","url":"index.html"},{"doc":"signal","title":"signal","content":"# How signal works\n\nSignal is a light event sourcing library that attempts to keep its interface simple, complexity low, and avoid any &quot;magical&quot; behavior.\n\nA signal service is composed of 3 core elements:\n\n-   An aggregate pool\n-   Aggregates\n-   A message bus\n\n---\n\n## Architecture\n\n**The Aggregate Pool** actor controls the aggregates in memory, and uses the storage handler to build them up from storage.\n\n**The Aggregates** are actors that use the user defined handlers to process commands, and output events (which also modify their state).\n\n**The Bus** actor forwards all events produced by all aggregates to subscribers, like a pub/sub.\n\nThe subscribers include the storage handler which persists events in a database. The subscribers can vary and do all kinds of things, more docs on that are coming.\n\n![alt text](how_signal_works.png)\n\n---\n\n## Consistency\n\nAn aggregate which processes a command is always represented by an actor in-memory, and the events produced out of a command are applied to actors state as a part of command execution. This makes **aggregates strongly consistent**. However, the same events are stored to a storage medium in an async manner, making the **storage eventually consistent**.\n\nWhat this means in practice is that when you process a command, and it returns Ok, your aggregates state is guaranteed to be the latest state. But the database might not be up to date right away.\n\n---\n\n## Delivery guarantees\n\nThe subscribers are informed of each event produced by the system as a result of processing a new command, they are not informed of events used by the Pool to hydrate an aggregate from the storage medium. As a rule of thumb this means that the Bus will inform the subscribers **exactly-once** for each event, however this is subject to standard Erlang BEAM VM message passing limitations.\n","url":"how-signal-works.html"},{"doc":"signal","title":"Aggregate","content":"pub type Aggregate(aggregate, command, event) =\n  process.Subject(AggregateMessage(aggregate, command, event))\n An aggregate is an actor managed by signal that holds the state, processes commands and events.\n \n You can send messasges to this aggregate and interact with it, but signal provides a number of pre-built functions to help with that.\n \n\nSynonyms:\nsignal.Aggregate\nsignal Aggregate","url":"signal.html#Aggregate"},{"doc":"signal","title":"AggregateConfig","content":"pub type AggregateConfig(aggregate, command, event) {\n  AggregateConfig(\n    initial_state: aggregate,\n    command_handler: CommandHandler(aggregate, command, event),\n    event_handler: EventHandler(aggregate, event),\n  )\n}\n Configures the aggregate processed by the signal service.\n\nAggregateConfig(\n  initial_state: aggregate,\n  command_handler: CommandHandler(aggregate, command, event),\n  event_handler: EventHandler(aggregate, event),\n)\n\n\nSynonyms:\nsignal.AggregateConfig\nsignal AggregateConfig","url":"signal.html#AggregateConfig"},{"doc":"signal","title":"AggregateMessage","content":"pub type AggregateMessage(aggregate, command, event) {\n  State(reply_with: process.Subject(aggregate))\n  Identity(reply_with: process.Subject(String))\n  HandleCommand(\n    reply_with: process.Subject(Result(aggregate, String)),\n    command,\n  )\n  ShutdownAggregate\n}\n\nState(reply_with: process.Subject(aggregate))\n\n\nIdentity(reply_with: process.Subject(String))\n\n\nHandleCommand(\n  reply_with: process.Subject(Result(aggregate, String)),\n  command,\n)\n\n\nShutdownAggregate\n\n\nSynonyms:\nsignal.AggregateMessage\nsignal AggregateMessage","url":"signal.html#AggregateMessage"},{"doc":"signal","title":"CommandHandler","content":"pub type CommandHandler(state, command, event) =\n  fn(command, state) -&gt; Result(List(event), String)\n This is where you put your domain / business logic, a function that has access to the current state of your aggregate, and can decided what to do with a given command.\n \n The command handler may return an Error(String), or a list of events. Most of the time, you will be producing one event but in some cases you will find a need for producing multiple.\n \n Commands are only triggered once, so they can contain side-effects.\n \n Basic example:\n \n pub fn handle_post_commands(command: PostCommands, post: Post) {\n   case command {\n     UpdatePostContent(title, text) -&gt; Ok([PostUpdated(title, text)])\n     PublishPost -&gt; {\n       case state {\n         s if s.title == &quot;&quot; -&gt; Error(&quot;Cannot publish a post without a title!&quot;)\n         _ -&gt; Ok([PostPublished()])\n       }\n     }\n   }\n } \n \n \n &gt; ⚠️ It is best practice to wrap the handler in a higher order function, this lets you inject dependencies and improves extensibility.\n Best practice:\n \n \n pub fn handle_post_commands(notify: NotificationService) -&gt; signal.CommandHandler(Post, PostCommands, PostEvents) {\n   fn (command: PostCommands, post: Post) {\n     case command {\n       UpdatePostContent(title, text) -&gt; Ok([PostUpdated(title, text)])\n       PublishPost -&gt; {\n         case state {\n           s if s.title == &quot;&quot; -&gt; Error(&quot;Cannot publish a post without a title!&quot;)\n           _ -&gt; { \n             notification.send(notify, &quot;Just published a new post - &quot; &lt;&gt; post.title)\n             Ok([PostPublished()]) }\n         }\n       }\n     }\n   }\n } \n \n \n\nSynonyms:\nsignal.CommandHandler\nsignal CommandHandler","url":"signal.html#CommandHandler"},{"doc":"signal","title":"ConsumerMessage","content":"pub type ConsumerMessage(state, event) {\n  Consume(Event(event))\n  GetConsumerState(reply: process.Subject(state))\n  ShutdownConsumer\n}\n Consumers are actors that should receive and handle these messages.\n \n - Consume: is the only message triggered by signal, and it is triggered on all events processed by the service\n \n Other messages are there for user convenience.\n \nConsume(Event(event))\n\n\nGetConsumerState(reply: process.Subject(state))\n\n\nShutdownConsumer\n\n\nSynonyms:\nsignal.ConsumerMessage\nsignal ConsumerMessage","url":"signal.html#ConsumerMessage"},{"doc":"signal","title":"Event","content":"pub type Event(event) {\n  Event(\n    aggregate_version: Int,\n    aggregate_id: String,\n    event_name: String,\n    data: event,\n  )\n}\n Represents a base event type that is used throughout signal, in event handlers you are able to use this information if needs be.\n \n\nEvent(\n  aggregate_version: Int,\n  aggregate_id: String,\n  event_name: String,\n  data: event,\n)\n\n\nSynonyms:\nsignal.Event\nsignal Event","url":"signal.html#Event"},{"doc":"signal","title":"EventHandler","content":"pub type EventHandler(state, event) =\n  fn(state, Event(event)) -&gt; state\n A function that describes how events translate into state of your aggregate. Events handlers are used to update the aggregate after processing commands, and to hydrate the aggregate from storage.\n \n Most of the time these are simple data mapping functions.\n \n Basic example:\n \n pub fn handle_post_events(post: Post, event: PostEvents) {\n   case command {\n     PostUpdated(title, text) -&gt; Post(..post, title: title, text: text)\n     PostPublished -&gt; Post(..post, published: True) \n   }\n } \n \n \n\nSynonyms:\nsignal.EventHandler\nsignal EventHandler","url":"signal.html#EventHandler"},{"doc":"signal","title":"LogLevel","content":"pub type LogLevel {\n  LogDebug\n  LogInfo\n  LogWarning\n  LogError\n}\n Just a basic log severity model\n \nLogDebug\n\n\nLogInfo\n\n\nLogWarning\n\n\nLogError\n\n\nSynonyms:\nsignal.LogLevel\nsignal LogLevel","url":"signal.html#LogLevel"},{"doc":"signal","title":"Signal","content":"pub type Signal(aggregate, command, event) =\n  process.Subject(SignalMessage(aggregate, command, event))\n A base signal process which supervises the behind the scenes stuff and exposes some functionality.\n \n\n\nSynonyms:\nsignal.Signal\nsignal Signal","url":"signal.html#Signal"},{"doc":"signal","title":"SignalConfig","content":"pub opaque type SignalConfig(aggregate, state, command, event)\n Configures the internals of an signal service.\n\n\nSynonyms:\nsignal.SignalConfig\nsignal SignalConfig","url":"signal.html#SignalConfig"},{"doc":"signal","title":"SignalMessage","content":"pub opaque type SignalMessage(aggregate, command, event)\n The base signal process, it handles the internal operation of signal for a given aggregate type.\n \n\n\nSynonyms:\nsignal.SignalMessage\nsignal SignalMessage","url":"signal.html#SignalMessage"},{"doc":"signal","title":"StoreMessage","content":"pub type StoreMessage(event) {\n  GetStoredEvents(\n    process.Subject(Result(List(Event(event)), String)),\n    String,\n  )\n  IsIdentityAvailable(\n    process.Subject(Result(Bool, String)),\n    String,\n  )\n  StoreEvent(Event(event))\n  ShutdownPersistanceLayer\n}\n When implementing a custom persistance layer, signal expects an actor that handles these messages\n \n - GetStoredEvents: used for hydrating aggregates from storage\n - IsIdentityAvailable: used to ensure duplicate ids cannot be created\n - StoreEvents: used to persist a list of new events\n - ShutdownPersistanceLayer: helper to let you shut down your actor, signal will not trigger this message\n \n &gt; ⚠️ Persistance actor has to report the result of the StoreEvents message in form of a PersistanceState event. This allows signal to handle a write ahead log and batch event storage operations. \n \nGetStoredEvents(\n  process.Subject(Result(List(Event(event)), String)),\n  String,\n)\n\n\nIsIdentityAvailable(\n  process.Subject(Result(Bool, String)),\n  String,\n)\n\n\nStoreEvent(Event(event))\n\n\nShutdownPersistanceLayer\n\n\nSynonyms:\nsignal.StoreMessage\nsignal StoreMessage","url":"signal.html#StoreMessage"},{"doc":"signal","title":"Subscriber","content":"pub type Subscriber(state, event) {\n  Consumer(process.Subject(ConsumerMessage(state, event)))\n  Policy(task.Task(Event(event)))\n}\n Subscribers are triggered on **all** events produced by all aggregates, and serve as a great way to extend your system.\n \n Subscribers cannot modify or produce events.\n \n These are generally great for creating different read models of your data, reporting, and reacting to certain events. \n \n  \n - Consumer: is an actor that consumes events, and can do whatever it wants with them, and give the user full control of the state, lifecycle and everything elese.\n - Policy: is a one-of task that should run on an event, at the moment there is no retries and signal will ignore the return values of these tasks.\n \n &gt; 🛑 Policies are early in development, not well tested and might result in performance bottlenecks.\n \nConsumer(process.Subject(ConsumerMessage(state, event)))\n\n\nPolicy(task.Task(Event(event)))\n\n\nSynonyms:\nsignal.Subscriber\nsignal Subscriber","url":"signal.html#Subscriber"},{"doc":"signal","title":"TelemetryEvent","content":"pub type TelemetryEvent {\n  PoolCreatingAggregate(aggregate_id: String)\n  PoolCreatedAggregate(aggregate_id: String)\n  PoolCannotCreateAggregateWithId(aggregate_id: String)\n  PoolHydratingAggregate(aggregate_id: String)\n  PoolHydratedAggregate(aggregate_id: String)\n  PoolAggregateNotFound(aggregate_id: String)\n  PoolRebalancingStarted(size: Int)\n  PoolEvictedAggregate(aggregate_id: String)\n  PoolRebalancingCompleted(new_size: Int)\n  AggregateProcessingCommand(\n    command_name: String,\n    aggregate_id: String,\n  )\n  AggregateProcessedCommand(\n    command_name: String,\n    aggregate_id: String,\n  )\n  AggregateCommandProcessingFailed(\n    command_name: String,\n    aggregate_id: String,\n  )\n  AggregateEventsProduced(\n    event_name: String,\n    aggregate_id: String,\n  )\n  BusTriggeringSubscribers(event_name: String, subscribers: Int)\n  BusSubscribersInformed(event_name: String, subscribers: Int)\n  StorePushedEventToWriteAheadLog(\n    event_name: String,\n    pool_size: Int,\n  )\n  StoreWriteAheadLogSizeWarning(pool_size: Int)\n  StoreSubmittedBatchForPersistance(batch_size: Int)\n  StorePersistanceCompleted(processed: Int, wal: Int)\n}\n Telemetry events produced by signal, these can be used for logging, metric collection and tracing\n \nPoolCreatingAggregate(aggregate_id: String)\n\n\nPoolCreatedAggregate(aggregate_id: String)\n\n\nPoolCannotCreateAggregateWithId(aggregate_id: String)\n\n\nPoolHydratingAggregate(aggregate_id: String)\n\n\nPoolHydratedAggregate(aggregate_id: String)\n\n\nPoolAggregateNotFound(aggregate_id: String)\n\n\nPoolRebalancingStarted(size: Int)\n\n\nPoolEvictedAggregate(aggregate_id: String)\n\n\nPoolRebalancingCompleted(new_size: Int)\n\n\nAggregateProcessingCommand(\n  command_name: String,\n  aggregate_id: String,\n)\n\n\nAggregateProcessedCommand(\n  command_name: String,\n  aggregate_id: String,\n)\n\n\nAggregateCommandProcessingFailed(\n  command_name: String,\n  aggregate_id: String,\n)\n\n\nAggregateEventsProduced(event_name: String, aggregate_id: String)\n\n\nBusTriggeringSubscribers(event_name: String, subscribers: Int)\n\n\nBusSubscribersInformed(event_name: String, subscribers: Int)\n\n\nStorePushedEventToWriteAheadLog(\n  event_name: String,\n  pool_size: Int,\n)\n\n\nStoreWriteAheadLogSizeWarning(pool_size: Int)\n\n\nStoreSubmittedBatchForPersistance(batch_size: Int)\n\n\nStorePersistanceCompleted(processed: Int, wal: Int)\n\n\nSynonyms:\nsignal.TelemetryEvent\nsignal TelemetryEvent","url":"signal.html#TelemetryEvent"},{"doc":"signal","title":"TelemetryMessage","content":"pub type TelemetryMessage {\n  Report(event: TelemetryEvent, template: String)\n  ShutdownTelemetry\n}\n Simple message interface for logging events.\n \n The template string is separated with | and is used to format the message.\n It can be populated with the telemetry event data using a helper function format_telemetry_message.\n \nReport(event: TelemetryEvent, template: String)\n\n\nShutdownTelemetry\n\n\nSynonyms:\nsignal.TelemetryMessage\nsignal TelemetryMessage","url":"signal.html#TelemetryMessage"},{"doc":"signal","title":"aggregate","content":"pub fn aggregate(\n  signal: Subject(SignalMessage(a, b, c)),\n  id: String,\n) -&gt; Result(Subject(AggregateMessage(a, b, c)), String)\n Use this function to retrieve a particular aggregate from the signal event store. This will return a subject which can then be used to interact with state of you aggregate, or process further commands.\n \n *Command handling example:*\n \n let result = signal.aggregate(em, &quot;how-to-gleam&quot;)\n |&gt; signal.handle_command(CommentOnPost(&quot;how-to-gleam&quot;))\n \n\n *Getting state example:*\n \n let post = signal.aggregate(em, &quot;how-to-gleam&quot;)\n |&gt; signal.get_state()\n \nSynonyms:\nsignal.aggregate\nsignal aggregate","url":"signal.html#aggregate"},{"doc":"signal","title":"configure","content":"pub fn configure(\n  agg: AggregateConfig(a, b, c),\n) -&gt; SignalConfig(a, d, b, c)\n This is a configuration object that lets you set up your signal instance.\n \n You should put the configuration somewhere in your app&#39;s startup code.\n \n \n let aggregate_config = AggregateConfig(\n   initial_state: my_default_aggregate,\n   command_handler: my_command_handler,\n   event_handler: my_event_handler\n )\n \n let store = signal.configure(aggregate_config)\n |&gt; signal.with_persistance_layer(my_storage)\n |&gt; signal.with_subscriber(my_notification_client)\n |&gt; signal.with_subscriber(my_metrics_counter)\n |&gt; signal.start()\n \nSynonyms:\nsignal.configure\nsignal configure","url":"signal.html#configure"},{"doc":"signal","title":"console_logger","content":"pub fn console_logger(\n  log_info: Bool,\n  log_debug: Bool,\n) -&gt; fn(TelemetryMessage, Nil) -&gt; Next(a, Nil)\n A simple console logger that logs telemetry events to the console.\n \nSynonyms:\nsignal.console_logger\nsignal console_logger","url":"signal.html#console_logger"},{"doc":"signal","title":"create","content":"pub fn create(\n  signal: Subject(SignalMessage(a, b, c)),\n  id: String,\n) -&gt; Result(Subject(AggregateMessage(a, b, c)), String)\n Creates a new aggregate with a given ID.\n \n The ID needs to be unique, otherwise creation will fail.\n \nSynonyms:\nsignal.create\nsignal create","url":"signal.html#create"},{"doc":"signal","title":"format_telemetry_message","content":"pub fn format_telemetry_message(\n  data: TelemetryEvent,\n  template: String,\n) -&gt; String\n Formats a telemetry message using a template string and telemetry event data.\n \nSynonyms:\nsignal.format_telemetry_message\nsignal format_telemetry_message","url":"signal.html#format_telemetry_message"},{"doc":"signal","title":"get_current_pool_size","content":"pub fn get_current_pool_size(\n  signal: Subject(SignalMessage(a, b, c)),\n) -&gt; Int\n Gets the current size of the aggregate pool in memory, mainly for testing.\n \nSynonyms:\nsignal.get_current_pool_size\nsignal get_current_pool_size","url":"signal.html#get_current_pool_size"},{"doc":"signal","title":"get_id","content":"pub fn get_id(agg: Subject(AggregateMessage(a, b, c))) -&gt; String\n Gets the id of the aggregate actor.\n \nSynonyms:\nsignal.get_id\nsignal get_id","url":"signal.html#get_id"},{"doc":"signal","title":"get_state","content":"pub fn get_state(agg: Subject(AggregateMessage(a, b, c))) -&gt; a\n Use this function to get the current state of your aggregate.\n \n \n let post = signal.aggregate(em, &quot;how-to-gleam&quot;)\n |&gt; signal.get_state()\n \nSynonyms:\nsignal.get_state\nsignal get_state","url":"signal.html#get_state"},{"doc":"signal","title":"handle_command","content":"pub fn handle_command(\n  agg: Subject(AggregateMessage(a, b, c)),\n  command: b,\n) -&gt; Result(a, String)\n Use this function to have your aggregate process a command.\n \n \n let result = signal.aggregate(em, &quot;how-to-gleam&quot;)\n |&gt; signal.handle_command(CreatePost(&quot;how-to-gleam&quot;))\n \n \nSynonyms:\nsignal.handle_command\nsignal handle_command","url":"signal.html#handle_command"},{"doc":"signal","title":"in_memory_persistance_handler","content":"pub fn in_memory_persistance_handler(\n  message: StoreMessage(a),\n  state: List(Event(a)),\n) -&gt; Next(b, List(Event(a)))\n Only public for testing purposes, you do not need to use this, it is a signal default.\n \nSynonyms:\nsignal.in_memory_persistance_handler\nsignal in_memory_persistance_handler","url":"signal.html#in_memory_persistance_handler"},{"doc":"signal","title":"start","content":"pub fn start(\n  config: SignalConfig(a, b, c, d),\n) -&gt; Result(Subject(SignalMessage(a, c, d)), StartError)\n Starts the signal services and returns a subject used to interact with the event store.\n \nSynonyms:\nsignal.start\nsignal start","url":"signal.html#start"},{"doc":"signal","title":"telemetry_log_level","content":"pub fn telemetry_log_level(ev: TelemetryEvent) -&gt; LogLevel\n Default logging level for telemetry events, when implementing a custom logger, you can use this to filter out events.\n \nSynonyms:\nsignal.telemetry_log_level\nsignal telemetry_log_level","url":"signal.html#telemetry_log_level"},{"doc":"signal","title":"with_custom_logger","content":"pub fn with_custom_logger(\n  config: SignalConfig(a, b, c, d),\n  logger: Subject(TelemetryMessage),\n) -&gt; SignalConfig(a, b, c, d)\n Allows for custom logging of telemetry events.\n \nSynonyms:\nsignal.with_custom_logger\nsignal with_custom_logger","url":"signal.html#with_custom_logger"},{"doc":"signal","title":"with_persistance_layer","content":"pub fn with_persistance_layer(\n  config: SignalConfig(a, b, c, d),\n  persist: Subject(StoreMessage(d)),\n) -&gt; SignalConfig(a, b, c, d)\n Configures signal to store events using a particular persistance layer.\n \n Signal will default to an **in-memory store** which is recommended for development.\n \n WIP - I am working on some persistance layers, but for now, you can bring your own, or play around with in-memory persistance.\n \nSynonyms:\nsignal.with_persistance_layer\nsignal with_persistance_layer","url":"signal.html#with_persistance_layer"},{"doc":"signal","title":"with_pool_size_limit","content":"pub fn with_pool_size_limit(\n  config: SignalConfig(a, b, c, d),\n  aggregates_in_memory: Int,\n) -&gt; SignalConfig(a, b, c, d)\n Defines the maximum number of aggregates kept in memory. **Defaults to 100**,\n lower it if you desire lower memory consumption, increase it if you desire higher performance.\n \n When an aggregate which is not in the pool is requested, signal has to rebuild it from events in the database.\n \n &gt; ⚠️ **Large aggregates** that contain a lot of data are an **anti-pattern** in event sourcing, instead of lowering the pool size,\n &gt; you might want to consider breaking up your aggregate and redesigning it, or storing some data using a different persistance method.\n \nSynonyms:\nsignal.with_pool_size_limit\nsignal with_pool_size_limit","url":"signal.html#with_pool_size_limit"},{"doc":"signal","title":"with_subscriber","content":"pub fn with_subscriber(\n  config: SignalConfig(a, b, c, d),\n  sub: Subscriber(b, d),\n) -&gt; SignalConfig(a, b, c, d)\n Subscribers can be one-of tasks (policies) or actors (consumers) that consume events generated by the aggregate.\n They are called for each event produced by all aggregates.\n  \n This is a great way of projecting state in a very specific way. Think of it as letting you create different read models of your data, or trigger some other specifics when an event is generated.\n\n You can even use this method to trigger commands to your other aggregates, but be careful, that can make it difficult to track the state of you application!\n\n *Example consumer:*\n \n fn event_counter( message: signal.ConsumerMessage(MyBlogEvent), event_count: Int ) {\n   case messasge {\n     ShutdownConsumer -&gt; actor.stop(process.Normal)\n     Consume(signal.Event(_)) -&gt; actor.continue(event_count + 1)\n     GetConsumerState(s) -&gt; {\n       process.send(s, event_count)\n       actor.continue(event_count)\n     }\n   }\n } \n \n There are a few things to note about Consumers:\n - Signal will not start or stop your consumers, their lifetime is in your control.\n - Signal will ignore any returned data\n - Your actor should accept the messages which are actually the Events you defined at configuration time\n \nSynonyms:\nsignal.with_subscriber\nsignal with_subscriber","url":"signal.html#with_subscriber"},{"doc":"signal","title":"without_debug_logging","content":"pub fn without_debug_logging(\n  config: SignalConfig(a, b, c, d),\n) -&gt; SignalConfig(a, b, c, d)\n Disables debug level logging for the **default logger**.\n \n &gt; ⚠️ this setting does not affect custom loggers.\n \nSynonyms:\nsignal.without_debug_logging\nsignal without_debug_logging","url":"signal.html#without_debug_logging"},{"doc":"signal","title":"without_info_logging","content":"pub fn without_info_logging(\n  config: SignalConfig(a, b, c, d),\n) -&gt; SignalConfig(a, b, c, d)\n Disables info level logging for the **default logger**.\n \n &gt; ⚠️ this setting does not affect custom loggers.\n \nSynonyms:\nsignal.without_info_logging\nsignal without_info_logging","url":"signal.html#without_info_logging"},{"doc":"signal","title":"signal","content":"","url":"signal.html"},{"doc":"signal/testing","title":"PersistanceTestEvent","content":"pub type PersistanceTestEvent {\n  SimpleEvent(String)\n  ComplexEvent(String, List(Int))\n}\n\nSimpleEvent(String)\n\n\nComplexEvent(String, List(Int))\n\n\nSynonyms:\nsignal/testing.PersistanceTestEvent\nsignal/testing PersistanceTestEvent","url":"signal/testing.html#PersistanceTestEvent"},{"doc":"signal/testing","title":"persistance_layer_complies_with_signal","content":"pub fn persistance_layer_complies_with_signal(\n  persistance_layer: Subject(StoreMessage(PersistanceTestEvent)),\n) -&gt; List(Result(Nil, String))\n\nSynonyms:\nsignal/testing.persistance_layer_complies_with_signal\nsignal/testing persistance_layer_complies_with_signal","url":"signal/testing.html#persistance_layer_complies_with_signal"},{"doc":"signal/testing","title":"signal/testing","content":"","url":"signal/testing.html"}]);